/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

/**
 * Used for <scale> to listen to slider changes to avoid mutation listeners
 */
[scriptable, uuid(e5b3074e-ee18-4538-83b9-2487d90a2a34)]
interface nsISliderListener : nsISupports
{
  /**
   * Called when the current, minimum or maximum value has been changed to
   * newValue. The which parameter will either be 'curpos', 'minpos' or 'maxpos'.
   * If userChanged is true, then the user changed ths slider, otherwise it
   * was changed via some other means.
   */
  void valueChanged(in AString which, in long newValue, in boolean userChanged);

  /**
   * Called when the user begins or ends dragging the thumb.
   */
  void dragStateChanged(in boolean isDragging);
};
