/* -*- Mode: IDL; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

[scriptable, uuid(85cd2640-e91e-41ac-bdca-1dbf10dc131e)]
interface nsIRedirectResultListener : nsISupports
{
  /**
   *  When an HTTP redirect has been processed (either successfully or not)
   *  nsIHttpChannel will call this function if its callbacks implement this
   *  interface.
   *
   *  @param proceeding
   *         Indicated whether the redirect will be proceeding, or not (i.e.
   *         has been canceled, or failed).
   */
  void onRedirectResult(in boolean proceeding);
};
