/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMWindow.idl"

[scriptable, uuid(6368a821-d3e2-4cbd-9699-5a8ba569e2f3)]
interface nsIDOMGlobalPropertyInitializer : nsISupports
{
  /*
   * Initialize the global property.
   *
   * @param window the global object on which the property is being retrieved.
   *
   * @returns a JS Object to use use as the property's value.
   */
  jsval init(in nsIDOMWindow window);
};
